use {
	gtk4 :: {
		Box , Image , Label , ListBox , ListBoxRow , Popover ,
		Align :: * ,
		Orientation :: * ,
		prelude :: * ,
	} ,
	libadwaita :: * ,
	std :: cell :: * ,
} ;

use crate :: ui :: { collated_grid :: * , component :: * , utility :: * } ;



pub struct FilmCollationMenu {
	widget : Box ,
	sorting : RefCell < ( FilmsSortedBy , SortingDirection ) > ,
}
pub struct SeriesCollationMenu { widget : Box }

impl FilmCollationMenu {
	pub fn new < F : Fn ( FilmsSortedBy , SortingDirection ) + 'static > ( on_sort : F ) -> Self {
		let widget = Box :: builder ()
			. orientation (Horizontal)
			. halign (Center)
			. spacing (20)
			. name ("film-collation-menu")
			. css_classes ( ["toolbar"] )
			. build () ;

		widget . append ( & create_sort_button ( & create_films_sort_menu (on_sort) ) ) ;
		widget . append ( & create_filter_button () ) ;

		let sorting = RefCell :: new ( ( FilmsSortedBy :: Name , SortingDirection :: Ascending ) ) ;

		Self { widget , sorting }
	}
}
impl SeriesCollationMenu {
	pub fn new < F : Fn (SeriesSortedBy) + 'static > ( on_sort : F ) -> Self {
		let widget = Box :: builder ()
			. orientation (Horizontal)
			. halign (Center)
			. spacing (20)
			. css_classes ( ["toolbar"] )
			. build () ;

		widget . append ( & create_sort_button ( & create_series_sort_menu (on_sort) ) ) ;
		widget . append ( & SplitButton :: builder () . label ("Filter") . build () ) ;

		Self { widget }
	}
}

impl Component <Box> for FilmCollationMenu {
	fn get_widget ( & self ) -> & Box { & self . widget }
}
impl Component <Box> for SeriesCollationMenu {
	fn get_widget ( & self ) -> & Box { & self . widget }
}

fn create_sort_button ( sort_menu : & Popover ) -> SplitButton {
	SplitButton :: builder ()
		. child ( & create_label ("Sort") )
		. popover (sort_menu)
		. build ()
}

fn create_films_sort_menu < F : Fn ( FilmsSortedBy , SortingDirection ) + 'static > ( on_sort : F ) -> Popover {
	let container = ListBox :: new () ;

	container . append ( & create_sort_menu_entry ( "Name" , false ) ) ;
	container . append ( & create_sort_menu_entry ( "Release date" , false ) ) ;
	container . append ( & create_sort_menu_entry ( "Runtime" , false ) ) ;

	container . select_row ( container . row_at_index (0) . as_ref () ) ;

	container . connect_row_activated ( move | _ , row | on_sort (
		match row . index () {
			0 => FilmsSortedBy :: Name ,
			1 => FilmsSortedBy :: ReleaseDate ,
			2 => FilmsSortedBy :: Runtime ,
			_ => panic ! () ,
		} ,
		SortingDirection :: Ascending ,
	) ) ;

	Popover :: builder ()
		. child ( & container )
		. css_classes ( ["menu"] )
		. build ()
}
fn create_series_sort_menu < F : Fn (SeriesSortedBy) + 'static > ( on_sort : F ) -> Popover {
	let container = ListBox :: new () ;

	container . append ( & create_sort_menu_entry ( "Name" , false ) ) ;
	container . append ( & create_sort_menu_entry ( "First release date" , false ) ) ;

	container . select_row ( container . row_at_index (0) . as_ref () ) ;

	container . connect_row_activated ( move | _ , row | on_sort ( match row . index () {
		0 => SeriesSortedBy :: Name ,
		1 => SeriesSortedBy :: FirstReleaseDate ,
		_ => panic ! () ,
	} ) ) ;

	Popover :: builder ()
		. child ( & container )
		. css_classes ( ["menu"] )
		. build ()
}

fn create_sort_menu_entry ( label : & str , reverse : bool ) -> ListBoxRow {
	let icon = match reverse {
		false => Image :: from_icon_name ("view-sort-ascending-symbolic") ,
		true => Image :: from_icon_name ("view-sort-descending-symbolic") ,
	} ;

	let container = create_horizontal_box ! (
		& Label :: builder ()
			. label (label)
			. hexpand (true)
			. halign (Start)
			. build () ,
		& icon ,
	) ;
	container . set_spacing (20) ;

	ListBoxRow :: builder () . child ( & container ) . build ()
}

fn create_filter_button () -> SplitButton {
	SplitButton :: builder () . label ("Filter") . build ()
}
